module.exports = function(generator) {

var u = generator.util;
var hb = generator.handlebars;

// map icon name -> glyph html hex (keys cannot contain regexp special chars)
// from https://github.com/FortAwesome/Font-Awesome/blob/v4.7.0/css/font-awesome.css
var glyphMap = {
'500px':'26e',
'address-book':'2b9',
'address-book-o':'2ba',
'address-card':'2bb',
'address-card-o':'2bc',
'adjust':'042',
'adn':'170',
'align-center':'037',
'align-justify':'039',
'align-left':'036',
'align-right':'038',
'amazon':'270',
'ambulance':'0f9',
'american-sign-language-interpreting':'2a3',
'anchor':'13d',
'android':'17b',
'angellist':'209',
'angle-double-down':'103',
'angle-double-left':'100',
'angle-double-right':'101',
'angle-double-up':'102',
'angle-down':'107',
'angle-left':'104',
'angle-right':'105',
'angle-up':'106',
'apple':'179',
'archive':'187',
'area-chart':'1fe',
'arrow-circle-down':'0ab',
'arrow-circle-left':'0a8',
'arrow-circle-o-down':'01a',
'arrow-circle-o-left':'190',
'arrow-circle-o-right':'18e',
'arrow-circle-o-up':'01b',
'arrow-circle-right':'0a9',
'arrow-circle-up':'0aa',
'arrow-down':'063',
'arrow-left':'060',
'arrow-right':'061',
'arrow-up':'062',
'arrows':'047',
'arrows-alt':'0b2',
'arrows-h':'07e',
'arrows-v':'07d',
'asl-interpreting':'2a3',
'assistive-listening-systems':'2a2',
'asterisk':'069',
'at':'1fa',
'audio-description':'29e',
'automobile':'1b9',
'backward':'04a',
'balance-scale':'24e',
'ban':'05e',
'bandcamp':'2d5',
'bank':'19c',
'bar-chart':'080',
'bar-chart-o':'080',
'barcode':'02a',
'bars':'0c9',
'bath':'2cd',
'bathtub':'2cd',
'battery':'240',
'battery-0':'244',
'battery-1':'243',
'battery-2':'242',
'battery-3':'241',
'battery-4':'240',
'battery-empty':'244',
'battery-full':'240',
'battery-half':'242',
'battery-quarter':'243',
'battery-three-quarters':'241',
'bed':'236',
'beer':'0fc',
'behance':'1b4',
'behance-square':'1b5',
'bell':'0f3',
'bell-o':'0a2',
'bell-slash':'1f6',
'bell-slash-o':'1f7',
'bicycle':'206',
'binoculars':'1e5',
'birthday-cake':'1fd',
'bitbucket':'171',
'bitbucket-square':'172',
'bitcoin':'15a',
'black-tie':'27e',
'blind':'29d',
'bluetooth':'293',
'bluetooth-b':'294',
'bold':'032',
'bolt':'0e7',
'bomb':'1e2',
'book':'02d',
'bookmark':'02e',
'bookmark-o':'097',
'braille':'2a1',
'briefcase':'0b1',
'btc':'15a',
'bug':'188',
'building':'1ad',
'building-o':'0f7',
'bullhorn':'0a1',
'bullseye':'140',
'bus':'207',
'buysellads':'20d',
'cab':'1ba',
'calculator':'1ec',
'calendar':'073',
'calendar-check-o':'274',
'calendar-minus-o':'272',
'calendar-o':'133',
'calendar-plus-o':'271',
'calendar-times-o':'273',
'camera':'030',
'camera-retro':'083',
'car':'1b9',
'caret-down':'0d7',
'caret-left':'0d9',
'caret-right':'0da',
'caret-square-o-down':'150',
'caret-square-o-left':'191',
'caret-square-o-right':'152',
'caret-square-o-up':'151',
'caret-up':'0d8',
'cart-arrow-down':'218',
'cart-plus':'217',
'cc':'20a',
'cc-amex':'1f3',
'cc-diners-club':'24c',
'cc-discover':'1f2',
'cc-jcb':'24b',
'cc-mastercard':'1f1',
'cc-paypal':'1f4',
'cc-stripe':'1f5',
'cc-visa':'1f0',
'certificate':'0a3',
'chain':'0c1',
'chain-broken':'127',
'check':'00c',
'check-circle':'058',
'check-circle-o':'05d',
'check-square':'14a',
'check-square-o':'046',
'chevron-circle-down':'13a',
'chevron-circle-left':'137',
'chevron-circle-right':'138',
'chevron-circle-up':'139',
'chevron-down':'078',
'chevron-left':'053',
'chevron-right':'054',
'chevron-up':'077',
'child':'1ae',
'chrome':'268',
'circle':'111',
'circle-o':'10c',
'circle-o-notch':'1ce',
'circle-thin':'1db',
'clipboard':'0ea',
'clock-o':'017',
'clone':'24d',
'close':'00d',
'cloud':'0c2',
'cloud-download':'0ed',
'cloud-upload':'0ee',
'cny':'157',
'code':'121',
'code-fork':'126',
'codepen':'1cb',
'codiepie':'284',
'coffee':'0f4',
'cog':'013',
'cogs':'085',
'columns':'0db',
'comment':'075',
'comment-o':'0e5',
'commenting':'27a',
'commenting-o':'27b',
'comments':'086',
'comments-o':'0e6',
'compass':'14e',
'compress':'066',
'connectdevelop':'20e',
'contao':'26d',
'copy':'0c5',
'copyright':'1f9',
'creative-commons':'25e',
'credit-card':'09d',
'credit-card-alt':'283',
'crop':'125',
'crosshairs':'05b',
'css3':'13c',
'cube':'1b2',
'cubes':'1b3',
'cut':'0c4',
'cutlery':'0f5',
'dashboard':'0e4',
'dashcube':'210',
'database':'1c0',
'deaf':'2a4',
'deafness':'2a4',
'dedent':'03b',
'delicious':'1a5',
'desktop':'108',
'deviantart':'1bd',
'diamond':'219',
'digg':'1a6',
'dollar':'155',
'dot-circle-o':'192',
'download':'019',
'dribbble':'17d',
'drivers-license':'2c2',
'drivers-license-o':'2c3',
'dropbox':'16b',
'drupal':'1a9',
'edge':'282',
'edit':'044',
'eercast':'2da',
'eject':'052',
'ellipsis-h':'141',
'ellipsis-v':'142',
'empire':'1d1',
'envelope':'0e0',
'envelope-o':'003',
'envelope-open':'2b6',
'envelope-open-o':'2b7',
'envelope-square':'199',
'envira':'299',
'eraser':'12d',
'etsy':'2d7',
'eur':'153',
'euro':'153',
'exchange':'0ec',
'exclamation':'12a',
'exclamation-circle':'06a',
'exclamation-triangle':'071',
'expand':'065',
'expeditedssl':'23e',
'external-link':'08e',
'external-link-square':'14c',
'eye':'06e',
'eye-slash':'070',
'eyedropper':'1fb',
'fa':'2b4',
'facebook':'09a',
'facebook-f':'09a',
'facebook-official':'230',
'facebook-square':'082',
'fast-backward':'049',
'fast-forward':'050',
'fax':'1ac',
'feed':'09e',
'female':'182',
'fighter-jet':'0fb',
'file':'15b',
'file-archive-o':'1c6',
'file-audio-o':'1c7',
'file-code-o':'1c9',
'file-excel-o':'1c3',
'file-image-o':'1c5',
'file-movie-o':'1c8',
'file-o':'016',
'file-pdf-o':'1c1',
'file-photo-o':'1c5',
'file-picture-o':'1c5',
'file-powerpoint-o':'1c4',
'file-sound-o':'1c7',
'file-text':'15c',
'file-text-o':'0f6',
'file-video-o':'1c8',
'file-word-o':'1c2',
'file-zip-o':'1c6',
'files-o':'0c5',
'film':'008',
'filter':'0b0',
'fire':'06d',
'fire-extinguisher':'134',
'firefox':'269',
'first-order':'2b0',
'flag':'024',
'flag-checkered':'11e',
'flag-o':'11d',
'flash':'0e7',
'flask':'0c3',
'flickr':'16e',
'floppy-o':'0c7',
'folder':'07b',
'folder-o':'114',
'folder-open':'07c',
'folder-open-o':'115',
'font':'031',
'font-awesome':'2b4',
'fonticons':'280',
'fort-awesome':'286',
'forumbee':'211',
'forward':'04e',
'foursquare':'180',
'free-code-camp':'2c5',
'frown-o':'119',
'futbol-o':'1e3',
'gamepad':'11b',
'gavel':'0e3',
'gbp':'154',
'ge':'1d1',
'gear':'013',
'gears':'085',
'genderless':'22d',
'get-pocket':'265',
'gg':'260',
'gg-circle':'261',
'gift':'06b',
'git':'1d3',
'git-square':'1d2',
'github':'09b',
'github-alt':'113',
'github-square':'092',
'gitlab':'296',
'gittip':'184',
'glass':'000',
'glide':'2a5',
'glide-g':'2a6',
'globe':'0ac',
'google':'1a0',
'google-plus':'0d5',
'google-plus-circle':'2b3',
'google-plus-official':'2b3',
'google-plus-square':'0d4',
'google-wallet':'1ee',
'graduation-cap':'19d',
'gratipay':'184',
'grav':'2d6',
'group':'0c0',
'h-square':'0fd',
'hacker-news':'1d4',
'hand-grab-o':'255',
'hand-lizard-o':'258',
'hand-o-down':'0a7',
'hand-o-left':'0a5',
'hand-o-right':'0a4',
'hand-o-up':'0a6',
'hand-paper-o':'256',
'hand-peace-o':'25b',
'hand-pointer-o':'25a',
'hand-rock-o':'255',
'hand-scissors-o':'257',
'hand-spock-o':'259',
'hand-stop-o':'256',
'handshake-o':'2b5',
'hard-of-hearing':'2a4',
'hashtag':'292',
'hdd-o':'0a0',
'header':'1dc',
'headphones':'025',
'heart':'004',
'heart-o':'08a',
'heartbeat':'21e',
'history':'1da',
'home':'015',
'hospital-o':'0f8',
'hotel':'236',
'hourglass':'254',
'hourglass-1':'251',
'hourglass-2':'252',
'hourglass-3':'253',
'hourglass-end':'253',
'hourglass-half':'252',
'hourglass-o':'250',
'hourglass-start':'251',
'houzz':'27c',
'html5':'13b',
'i-cursor':'246',
'id-badge':'2c1',
'id-card':'2c2',
'id-card-o':'2c3',
'ils':'20b',
'image':'03e',
'imdb':'2d8',
'inbox':'01c',
'indent':'03c',
'industry':'275',
'info':'129',
'info-circle':'05a',
'inr':'156',
'instagram':'16d',
'institution':'19c',
'internet-explorer':'26b',
'intersex':'224',
'ioxhost':'208',
'italic':'033',
'joomla':'1aa',
'jpy':'157',
'jsfiddle':'1cc',
'key':'084',
'keyboard-o':'11c',
'krw':'159',
'language':'1ab',
'laptop':'109',
'lastfm':'202',
'lastfm-square':'203',
'leaf':'06c',
'leanpub':'212',
'legal':'0e3',
'lemon-o':'094',
'level-down':'149',
'level-up':'148',
'life-bouy':'1cd',
'life-ring':'1cd',
'life-saver':'1cd',
'lightbulb-o':'0eb',
'line-chart':'201',
'link':'0c1',
'linkedin':'0e1',
'linkedin-square':'08c',
'linode':'2b8',
'linux':'17c',
'list':'03a',
'list-alt':'022',
'list-ol':'0cb',
'list-ul':'0ca',
'location-arrow':'124',
'lock':'023',
'long-arrow-down':'175',
'long-arrow-left':'177',
'long-arrow-right':'178',
'long-arrow-up':'176',
'low-vision':'2a8',
'magic':'0d0',
'magnet':'076',
'mail-forward':'064',
'mail-reply':'112',
'mail-reply-all':'122',
'male':'183',
'map':'279',
'map-marker':'041',
'map-o':'278',
'map-pin':'276',
'map-signs':'277',
'mars':'222',
'mars-double':'227',
'mars-stroke':'229',
'mars-stroke-h':'22b',
'mars-stroke-v':'22a',
'maxcdn':'136',
'meanpath':'20c',
'medium':'23a',
'medkit':'0fa',
'meetup':'2e0',
'meh-o':'11a',
'mercury':'223',
'microchip':'2db',
'microphone':'130',
'microphone-slash':'131',
'minus':'068',
'minus-circle':'056',
'minus-square':'146',
'minus-square-o':'147',
'mixcloud':'289',
'mobile':'10b',
'mobile-phone':'10b',
'modx':'285',
'money':'0d6',
'moon-o':'186',
'mortar-board':'19d',
'motorcycle':'21c',
'mouse-pointer':'245',
'music':'001',
'navicon':'0c9',
'neuter':'22c',
'newspaper-o':'1ea',
'object-group':'247',
'object-ungroup':'248',
'odnoklassniki':'263',
'odnoklassniki-square':'264',
'opencart':'23d',
'openid':'19b',
'opera':'26a',
'optin-monster':'23c',
'outdent':'03b',
'pagelines':'18c',
'paint-brush':'1fc',
'paper-plane':'1d8',
'paper-plane-o':'1d9',
'paperclip':'0c6',
'paragraph':'1dd',
'paste':'0ea',
'pause':'04c',
'pause-circle':'28b',
'pause-circle-o':'28c',
'paw':'1b0',
'paypal':'1ed',
'pencil':'040',
'pencil-square':'14b',
'pencil-square-o':'044',
'percent':'295',
'phone':'095',
'phone-square':'098',
'photo':'03e',
'picture-o':'03e',
'pie-chart':'200',
'pied-piper':'2ae',
'pied-piper-alt':'1a8',
'pied-piper-pp':'1a7',
'pinterest':'0d2',
'pinterest-p':'231',
'pinterest-square':'0d3',
'plane':'072',
'play':'04b',
'play-circle':'144',
'play-circle-o':'01d',
'plug':'1e6',
'plus':'067',
'plus-circle':'055',
'plus-square':'0fe',
'plus-square-o':'196',
'podcast':'2ce',
'power-off':'011',
'print':'02f',
'product-hunt':'288',
'puzzle-piece':'12e',
'qq':'1d6',
'qrcode':'029',
'question':'128',
'question-circle':'059',
'question-circle-o':'29c',
'quora':'2c4',
'quote-left':'10d',
'quote-right':'10e',
'ra':'1d0',
'random':'074',
'ravelry':'2d9',
'rebel':'1d0',
'recycle':'1b8',
'reddit':'1a1',
'reddit-alien':'281',
'reddit-square':'1a2',
'refresh':'021',
'registered':'25d',
'remove':'00d',
'renren':'18b',
'reorder':'0c9',
'repeat':'01e',
'reply':'112',
'reply-all':'122',
'resistance':'1d0',
'retweet':'079',
'rmb':'157',
'road':'018',
'rocket':'135',
'rotate-left':'0e2',
'rotate-right':'01e',
'rouble':'158',
'rss':'09e',
'rss-square':'143',
'rub':'158',
'ruble':'158',
'rupee':'156',
's15':'2cd',
'safari':'267',
'save':'0c7',
'scissors':'0c4',
'scribd':'28a',
'search':'002',
'search-minus':'010',
'search-plus':'00e',
'sellsy':'213',
'send':'1d8',
'send-o':'1d9',
'server':'233',
'share':'064',
'share-alt':'1e0',
'share-alt-square':'1e1',
'share-square':'14d',
'share-square-o':'045',
'shekel':'20b',
'sheqel':'20b',
'shield':'132',
'ship':'21a',
'shirtsinbulk':'214',
'shopping-bag':'290',
'shopping-basket':'291',
'shopping-cart':'07a',
'shower':'2cc',
'sign-in':'090',
'sign-language':'2a7',
'sign-out':'08b',
'signal':'012',
'signing':'2a7',
'simplybuilt':'215',
'sitemap':'0e8',
'skyatlas':'216',
'skype':'17e',
'slack':'198',
'sliders':'1de',
'slideshare':'1e7',
'smile-o':'118',
'snapchat':'2ab',
'snapchat-ghost':'2ac',
'snapchat-square':'2ad',
'snowflake-o':'2dc',
'soccer-ball-o':'1e3',
'sort':'0dc',
'sort-alpha-asc':'15d',
'sort-alpha-desc':'15e',
'sort-amount-asc':'160',
'sort-amount-desc':'161',
'sort-asc':'0de',
'sort-desc':'0dd',
'sort-down':'0dd',
'sort-numeric-asc':'162',
'sort-numeric-desc':'163',
'sort-up':'0de',
'soundcloud':'1be',
'space-shuttle':'197',
'spinner':'110',
'spoon':'1b1',
'spotify':'1bc',
'square':'0c8',
'square-o':'096',
'stack-exchange':'18d',
'stack-overflow':'16c',
'star':'005',
'star-half':'089',
'star-half-empty':'123',
'star-half-full':'123',
'star-half-o':'123',
'star-o':'006',
'steam':'1b6',
'steam-square':'1b7',
'step-backward':'048',
'step-forward':'051',
'stethoscope':'0f1',
'sticky-note':'249',
'sticky-note-o':'24a',
'stop':'04d',
'stop-circle':'28d',
'stop-circle-o':'28e',
'street-view':'21d',
'strikethrough':'0cc',
'stumbleupon':'1a4',
'stumbleupon-circle':'1a3',
'subscript':'12c',
'subway':'239',
'suitcase':'0f2',
'sun-o':'185',
'superpowers':'2dd',
'superscript':'12b',
'support':'1cd',
'table':'0ce',
'tablet':'10a',
'tachometer':'0e4',
'tag':'02b',
'tags':'02c',
'tasks':'0ae',
'taxi':'1ba',
'telegram':'2c6',
'television':'26c',
'tencent-weibo':'1d5',
'terminal':'120',
'text-height':'034',
'text-width':'035',
'th':'00a',
'th-large':'009',
'th-list':'00b',
'themeisle':'2b2',
'thermometer':'2c7',
'thermometer-0':'2cb',
'thermometer-1':'2ca',
'thermometer-2':'2c9',
'thermometer-3':'2c8',
'thermometer-4':'2c7',
'thermometer-empty':'2cb',
'thermometer-full':'2c7',
'thermometer-half':'2c9',
'thermometer-quarter':'2ca',
'thermometer-three-quarters':'2c8',
'thumb-tack':'08d',
'thumbs-down':'165',
'thumbs-o-down':'088',
'thumbs-o-up':'087',
'thumbs-up':'164',
'ticket':'145',
'times':'00d',
'times-circle':'057',
'times-circle-o':'05c',
'times-rectangle':'2d3',
'times-rectangle-o':'2d4',
'tint':'043',
'toggle-down':'150',
'toggle-left':'191',
'toggle-off':'204',
'toggle-on':'205',
'toggle-right':'152',
'toggle-up':'151',
'trademark':'25c',
'train':'238',
'transgender':'224',
'transgender-alt':'225',
'trash':'1f8',
'trash-o':'014',
'tree':'1bb',
'trello':'181',
'tripadvisor':'262',
'trophy':'091',
'truck':'0d1',
'try':'195',
'tty':'1e4',
'tumblr':'173',
'tumblr-square':'174',
'turkish-lira':'195',
'tv':'26c',
'twitch':'1e8',
'twitter':'099',
'twitter-square':'081',
'umbrella':'0e9',
'underline':'0cd',
'undo':'0e2',
'universal-access':'29a',
'university':'19c',
'unlink':'127',
'unlock':'09c',
'unlock-alt':'13e',
'unsorted':'0dc',
'upload':'093',
'usb':'287',
'usd':'155',
'user':'007',
'user-circle':'2bd',
'user-circle-o':'2be',
'user-md':'0f0',
'user-o':'2c0',
'user-plus':'234',
'user-secret':'21b',
'user-times':'235',
'users':'0c0',
'vcard':'2bb',
'vcard-o':'2bc',
'venus':'221',
'venus-double':'226',
'venus-mars':'228',
'viacoin':'237',
'viadeo':'2a9',
'viadeo-square':'2aa',
'video-camera':'03d',
'vimeo':'27d',
'vimeo-square':'194',
'vine':'1ca',
'vk':'189',
'volume-control-phone':'2a0',
'volume-down':'027',
'volume-off':'026',
'volume-up':'028',
'warning':'071',
'wechat':'1d7',
'weibo':'18a',
'weixin':'1d7',
'whatsapp':'232',
'wheelchair':'193',
'wheelchair-alt':'29b',
'wifi':'1eb',
'wikipedia-w':'266',
'window-close':'2d3',
'window-close-o':'2d4',
'window-maximize':'2d0',
'window-minimize':'2d1',
'window-restore':'2d2',
'windows':'17a',
'won':'159',
'wordpress':'19a',
'wpbeginner':'297',
'wpexplorer':'2de',
'wpforms':'298',
'wrench':'0ad',
'xing':'168',
'xing-square':'169',
'y-combinator':'23b',
'y-combinator-square':'1d4',
'yahoo':'19e',
'yc':'23b',
'yc-square':'1d4',
'yelp':'1e9',
'yen':'157',
'yoast':'2b1',
'youtube':'167',
'youtube-play':'16a',
'youtube-square':'166'
};

function sGlyph(s) {
  return '&#xf' + s + ';'
}

// lookup glyph given name
hb.registerHelper('faGlyph', function(name, frame) {
  var glyph = glyphMap[name];
  return glyph ? sGlyph(glyph) : '';
});

// get icon html for a name - xtra classes optional
hb.registerHelper('faIcon', function(name, xtra, frame) {
  return iconHtml(name, hb.hbp(xtra));
});

// block helper over all {name: glyph:} sorted by glyph
hb.registerHelper('eachFa', function(frame) {
  return u.map(u.sortBy(u.toPairs(glyphMap), pair => pair[1]), pair => {
    return frame.fn({ name:pair[0], glyph:sGlyph(pair[1]) });
  }).join('');
});

// get html for name, with optional extra classes
function iconHtml(name, xtra) {
  var glyph = glyphMap[name];
  if (!glyph) return;
  var xtra = xtra ? (' fa-' + xtra.split(/\s/).join(' fa-')) : '';
  return '<span class="fa' + xtra + '">' + sGlyph(glyph) + '</span>';
}

// test for !glyphMapkey class1 class2 class3 ...
var re = new RegExp('^!(' + u.keys(glyphMap).join('|') + ')(?:\\s+(.+)|$)');

// marked plugin renderer function
function renderEm(text) {
  var match;
  if (match = u.str(text).match(re)) return iconHtml(match[1], match[2]);
  return false;
}

generator.marked.use( { renderer: { em:renderEm } } );
}
